@@ -8,7 +8,7 @@ import json
 from itertools import count
 from urllib.parse import parse_qs, urlparse
 import os
-from typing import Optional
+from typing import Optional, List, Tuple
 from enum import Enum
 
 import trio
@@ -56,6 +56,7 @@ class ResponseConfig:
     json_path: str = None
     throttle: bool = False
     force_retry: bool = False
+    request_headers: Optional[List[Tuple[bytes, bytes]]] = None
 
     def _resolve_file_path(self, wrapper, request_type):
         global SHOULD_THROTTLE
@@ -85,6 +86,16 @@ class ResponseConfig:
                      ".\n generate_body_size: ", self.generate_body_size)
         with open(self.json_path, 'r') as f:
             data = json.load(f)
+        headers = wrapper.basic_headers()
+
+        # If request_headers is present, validate that the request contains all required headers
+        if 'request_headers' in data:
+            for header in data['request_headers']:
+                header_bytes = header.encode('utf-8')
+                if not any(header_bytes == h[0] for h in self.request_headers):
+                    response = Response(status_code=500, delay=0, headers=headers,
+                                        data=json.dumps({'error': f"Missing required header: {header}"}), chunked=chunked, head_request=head_request)
+                    return response
 
         # if response has delay, then sleep before sending it
         delay = data.get('delay', 0)
@@ -95,7 +106,6 @@ class ResponseConfig:
         else:
             body = "\n".join(data['body'])
 
-        headers = wrapper.basic_headers()
         content_length_set = False
         for header in data['headers'].items():
             headers.append((header[0], str(header[1])))
@@ -401,7 +411,6 @@ def handle_get_object_modified(start_range, end_range, request):
     else:
         # Check the request header to make sure "If-Match" is set
         etag = get_request_header_value(request, "if-match")
-        print(etag)
         # fetch Etag from the first_part response file
         response_file = os.path.join(
             base_dir, S3Opts.GetObject.name, f"get_object_modified_first_part.json")
@@ -424,6 +433,12 @@ def handle_get_object(wrapper, request, parsed_path, head_request=False):
     else:
         RETRY_REQUEST_COUNT = 0
 
+    if (parsed_path.path == "/get_object_invalid_response_missing_content_range" or
+        parsed_path.path == "/get_object_invalid_response_missing_etags" or
+            parsed_path.path == "/get_object_long_error"):
+        # Don't generate the body for those requests
+        return response_config
+
     body_range_value = get_request_header_value(request, "range")
 
     if body_range_value:
@@ -439,9 +454,6 @@ def handle_get_object(wrapper, request, parsed_path, head_request=False):
 
     if parsed_path.path == "/get_object_modified":
         return handle_get_object_modified(start_range, end_range, request)
-    elif parsed_path.path == "/get_object_invalid_response_missing_content_range" or parsed_path.path == "/get_object_invalid_response_missing_etags":
-        # Don't generate the body for those requests
-        return response_config
 
     response_config.generate_body_size = data_length
     return response_config
@@ -496,6 +508,7 @@ async def handle_mock_s3_request(wrapper, request):
 
     if response_config is None:
         response_config = ResponseConfig(parsed_path.path)
+    response_config.request_headers = request.headers
 
     response = response_config.resolve_response(
         wrapper, request_type, head_request=method == "HEAD")
